package fr.amap.commons.util.swing.jprogress;

/**
 * Simple class to create progression event during a process, 
 * that can be used with a JProgressListener.
 * @see JProgressListener
 */
public final class JProgressEvent {
    /**
     * The progression value, must be between zero and the max value.
     */
    private final int value;
    
    /**
     * The max progression value.
     */
    private final int max;
    
    /**
     * The message to display for this event.
     */
    private final String message;

    /**
     * Create a progress event with full values.
     * @param value
     * @param max
     * @param message 
     */
    public JProgressEvent(final int value, final int max, final String message) {
        this.value = value;
        this.max = max;
        this.message = message;
    }
    
    /**
     * Create a progress by specifying the values, 
     * and set the message by default to "Loading, please wait...".
     * @param value
     * @param max 
     */
    public JProgressEvent(final int value, final int max) {
        this.value = value;
        this.max = max;
        this.message = "Loading, please wait...";
    }

    /**
     * The progression value, must be between zero and the max value.
     * @return 
     */
    public final int getValue() {
        return value;
    }

    /**
     * The max progression value.
     * @return 
     */
    public final int getMax() {
        return max;
    }

    /**
     * The message to display for this event.
     * @return 
     */
    public final String getMessage() {
        return message;
    }
}
