/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */

package fr.amap.lidar.amapvox.nephos3d.views;

import fr.amap.lidar.amapvox.nephos3d.util.InputStatus;
import com.jogamp.newt.awt.NewtCanvasAWT;
import com.jogamp.newt.event.KeyAdapter;
import com.jogamp.newt.event.KeyEvent;
import com.jogamp.newt.event.MouseAdapter;
import com.jogamp.newt.event.MouseEvent;
import com.jogamp.newt.event.WindowAdapter;
import com.jogamp.newt.event.WindowEvent;
import com.jogamp.newt.opengl.GLWindow;
import com.jogamp.opengl.GLCapabilities;
import com.jogamp.opengl.GLContext;
import com.jogamp.opengl.GLProfile;
import com.jogamp.opengl.util.Animator;
import java.awt.BorderLayout;
import java.awt.Panel;
import net.jcip.annotations.GuardedBy;
import net.jcip.annotations.ThreadSafe;

/**
 * OpenGL canvas to perform OpenGL processes.
 */
@ThreadSafe
public final class NephosPanel extends Panel {
    
    @GuardedBy(value = "this")
    private final GLWindow glWindow;
    
    @GuardedBy(value = "this")
    private final NephosGLEventListener glEventListener;
    
    public NephosPanel() {
        super();
        
        final GLCapabilities caps = new GLCapabilities(
                GLProfile.get(GLProfile.GL3bc));
        
        this.glWindow = GLWindow.create(caps);
        glWindow.setContextCreationFlags(GLContext.CTX_OPTION_DEBUG);
        glWindow.setUndecorated(false);
        glWindow.setPointerVisible(true);
        
        glEventListener = new NephosGLEventListener();
        
        glWindow.addWindowListener(new WindowAdapter() {
            @Override
            public void windowDestroyed(WindowEvent e) {
                dispose();
            }
        });
        
        glWindow.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e) {
                glEventListener.inputting(e, InputStatus.MOUSE_CLICKED);
            }

            @Override
            public void mouseDragged(MouseEvent e) {
                glEventListener.inputting(e, InputStatus.MOUSE_DRAGGED);
            }

            @Override
            public void mouseEntered(MouseEvent e) {
                glEventListener.inputting(e, InputStatus.MOUSE_ENTERED);
            }

            @Override
            public void mouseExited(MouseEvent e) {
                glEventListener.inputting(e, InputStatus.MOUSE_EXITED);
            }

            @Override
            public void mouseMoved(MouseEvent e) {
                glEventListener.inputting(e, InputStatus.MOUSE_MOVED);
            }

            @Override
            public void mouseReleased(MouseEvent e) {
                glEventListener.inputting(e, InputStatus.MOUSE_RELEASED);
            }

            @Override
            public void mouseWheelMoved(MouseEvent e) {
                glEventListener.inputting(e, InputStatus.MOUSE_WHEEL_MOVED);
            }
        });
        
        glWindow.addKeyListener(new KeyAdapter() {
            @Override
            public void keyPressed(KeyEvent e) {
                glEventListener.inputting(e, InputStatus.BUTTON_PRESSED);
            }

            @Override
            public void keyReleased(KeyEvent e) {
                glEventListener.inputting(e, InputStatus.BUTTON_RELEASED);
            }
        });
        
        glWindow.addGLEventListener(glEventListener);
        
        final NewtCanvasAWT canvas = new NewtCanvasAWT(glWindow);
        setLayout(new BorderLayout());
        add(canvas, BorderLayout.CENTER);
                
        final Animator animator = new Animator();
        animator.add(glWindow);
        animator.start();
    }

    public final synchronized NephosGLEventListener getGLEventListener() {
        return glEventListener;
    }
    
    public final synchronized void dispose() {
        glWindow.getAnimator().stop();
    }
    
}
