/* Uniform area */
uniform sampler2D screenTexture;

/* Output color */
out vec4 color;

const float offset = 0.001;  

void main(void)
{
    vec2 offsets[9] = vec2[](
        vec2(-offset,  offset), // top-left
        vec2( 0.0f,    offset), // top-center
        vec2( offset,  offset), // top-right
        vec2(-offset,  0.0f),   // center-left
        vec2( 0.0f,    0.0f),   // center-center
        vec2( offset,  0.0f),   // center-right
        vec2(-offset, -offset), // bottom-left
        vec2( 0.0f,   -offset), // bottom-center
        vec2( offset, -offset)  // bottom-right    
    );

    float amount = 0.05;

    float kernel[9] = float[]
    (
        -amount, -amount, -amount,
        -amount, 8 * amount + 1, -amount,
        -amount, -amount, -amount
    );

    vec3 sampleTex[9];
    
    for(int i = 0; i < 9; i++)
    {
        sampleTex[i] = texture(screenTexture, viewSpaceTexcoord.st + offsets[i]).rgb;
    }

    vec3 screenColor = vec3(0.0);
    for(int i = 0; i < 9; i++)
    {
        screenColor += sampleTex[i] * kernel[i];
    }

    float alpha = texture(screenTexture, viewSpaceTexcoord).a;
    color = vec4(screenColor, alpha);
}